#!/bin/bash

echo -e "\n[$(date +'%Y-%m-%d %H:%M:%S')] Starting setup..."
echo -e "\nStorage profile:\n$(df -h)"
echo -e "\nDocker profile:\n$(sudo docker system df)"
echo -e "\nDocker containers profile:\n$(sudo docker ps)"

FORCE="$1"

# Check if required dependencies exist
command_exists() {
    command -v "$@" >/dev/null
}

# Check if a service is active, if not try to start it
is_active() {
    local service="$1"
    echo "Checking for $service service..."
    if sudo systemctl is-active "$service" >/dev/null; then
        echo "$service service is running."
    else
        echo "$service service is not running. Trying to restart..."
        sudo systemctl start "$service"
        if sudo systemctl is-active "$service" >/dev/null; then
            echo "Successfully started service $service."
        else
            echo "Failed to start service $service."
            exit 1
        fi
    fi
}

# Update existing variables in env files
update_env_variable() {
    local variable_name="$1"
    local variable_value="$2"
    local file_path="$3"

    # Use sed to replace the value of the variable in the environment file
    sed -i "s|^$variable_name=.*|$variable_name=$variable_value|" "$file_path"
}

# Replace variable
replace_variable() {
    local variable_name="$1"
    local variable_value="$2"
    local file_path="$3"

    # Replace variable in the file
    sed -i "s/$variable_name/$variable_value/g" "$file_path"
}

extract_env_value_or_exit() {
    local variable_name="$1"
    local file_name="$2"

    # Use grep to find the line containing the variable and cut to extract its value
    local variable_value=$(grep "^$variable_name=" "$file_name" | cut -d '=' -f 2- | head -n 1)

    if [ -n "$variable_value" ]; then
        echo "$variable_value"
    else
        echo "Variable $variable_name not found in $file_name." >&2
        exit 1
    fi
}

ensure_dependencies() {
    # Install cron if it's not installed
    echo "Ensuring dependencies..."
    echo "Checking for crontab..."
    if ! command_exists crontab; then
        echo "Crontab is not installed. Installing..."
        sudo apt-get update
        sudo apt-get install -y cron
    else
        echo "Crontab is installed."
    fi

    # Install git if it's not installed
    echo "Checking for git..."
    if ! command_exists git; then
        echo "Git is not installed. Installing..."
        sudo apt-get update
        sudo apt-get install -y git
    else
        echo "Git is installed."
    fi

    # Install caddy if it's not installed
    echo "Checking for caddy..."
    if ! command_exists caddy; then
        echo "Caddy is not installed. Installing..."
        sudo apt-get install -y debian-keyring debian-archive-keyring apt-transport-https
        curl -1sLf 'https://dl.cloudsmith.io/public/caddy/stable/gpg.key' | sudo gpg --dearmor -o /usr/share/keyrings/caddy-stable-archive-keyring.gpg
        curl -1sLf 'https://dl.cloudsmith.io/public/caddy/stable/debian.deb.txt' | sudo tee /etc/apt/sources.list.d/caddy-stable.list
        sudo apt-get update
        sudo apt-get install -y caddy
    else
        echo "Caddy is installed."
    fi

    # Install docker if it's not installed
    echo "Checking for docker..."
    if ! command_exists docker; then
        echo "Docker is not installed. Installing..."
        sudo apt-get update
        sudo apt-get install -y apt-transport-https ca-certificates curl software-properties-common
        sudo curl -fsSL https://download.docker.com/linux/debian/gpg -o /etc/apt/keyrings/docker.asc
        sudo chmod a+r /etc/apt/keyrings/docker.asc
        echo \
            "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.asc] https://download.docker.com/linux/debian \
            $(. /etc/os-release && echo "$VERSION_CODENAME") stable" | \
            sudo tee /etc/apt/sources.list.d/docker.list > /dev/null
        sudo apt-get update
        sudo apt-get install -y docker-ce docker-ce-cli containerd.io docker-buildx-plugin docker-compose-plugin
        sudo systemctl restart docker
    else
        echo "Docker is installed."
    fi
}

ensure_dependencies

# Variables
PROJECT_PATH="/home/admin/shepherd-enterprise"
GIT_BRANCH="main"

ENV_SHEPHERD_PATH="$PROJECT_PATH/.env.shepherd"
ENV_CLIENT_PATH="$PROJECT_PATH/.env.client"
ENV_PATH="$PROJECT_PATH/.env"
ENV_ALERTMANAGER_URL_PATH="$PROJECT_PATH/.alertmanager-url"
ENV_ALERTMANAGER_SECRET_PATH="$PROJECT_PATH/.alertmanager-secret"
ENV_VIMACOG_GEN="$PROJECT_PATH/vimacog/.env-gen"
ENV_VIMACOG_HARD_GEN="$PROJECT_PATH/vimacog/.env-hard-gen"

CADDYFILE_PATH="$PROJECT_PATH/Caddyfile"

GRAFANADATA_PATH="$PROJECT_PATH/grafanadata"
LOKIDATA_PATH="$PROJECT_PATH/lokidata"

GIT_PATH="$PROJECT_PATH/.git"

SCRIPT_PATH="$PROJECT_PATH/setup-debian.sh"
SCRIPT_LOG_PATH="$PROJECT_PATH/cron-logs/setup-debian-cron.log"
CRON_JOB="0 0 * * * $SCRIPT_PATH >> $SCRIPT_LOG_PATH 2>&1"
# CRON_JOB="*/1 * * * * $SCRIPT_PATH >> $SCRIPT_LOG_PATH 2>&1"
CRON_EXISTS=$(crontab -l | grep -F "$SCRIPT_PATH" | grep -v grep | wc -l)

# NOTE: If this path is changed, change in `./vimacog/vimacog-gen.sh`
VIMACOG_GEN_PATH="$PROJECT_PATH/vimacog/vimacog-gen.sh"
VIMACOG_GEN_LOG_PATH="$PROJECT_PATH/cron-logs/vimacog-gen-cron.log"
VIMACOG_GEN_JOB="*/1 * * * * $VIMACOG_GEN_PATH >> $VIMACOG_GEN_LOG_PATH 2>&1"
VIMACOG_GEN_EXISTS=$(crontab -l | grep -F "$VIMACOG_GEN_PATH" | grep -v grep | wc -l)
VIMACOG_GEN_LOG_DIR="$PROJECT_PATH/vimacog/gen-logs/"
VIMACOG_GEN_BACKUP_DIR="$PROJECT_PATH/vimacog/gen-backup/"

# NOTE: If this path is changed, change in `./vimacog/vimacog-hard-gen.sh`
VIMACOG_HARD_GEN_PATH="$PROJECT_PATH/vimacog/vimacog-hard-gen.sh"
VIMACOG_HARD_GEN_LOG_PATH="$PROJECT_PATH/cron-logs/vimacog-hard-gen-cron.log"
VIMACOG_HARD_GEN_JOB="0 0 * * * $VIMACOG_HARD_GEN_PATH >> $VIMACOG_HARD_GEN_LOG_PATH 2>&1"
VIMACOG_HARD_GEN_EXISTS=$(crontab -l | grep -F "$VIMACOG_HARD_GEN_PATH" | grep -v grep | wc -l)
VIMACOG_HARD_GEN_LOG_DIR="$PROJECT_PATH/vimacog/hard-gen-logs/"
VIMACOG_HARD_GEN_BACKUP_DIR="$PROJECT_PATH/vimacog/hard-gen-backup/"

ONEAPI_LOG_PATH="$PROJECT_PATH/logs/oneapi"
ACCUMULATOR_LOG_PATH="$PROJECT_PATH/logs/reports-accumulator"

# Note: Cleanup temporary storgae
cleanup_git() {
    echo "Initiating git cleanup..."
    cd "$PROJECT_PATH" || exit
    sudo du -hc $GIT_PATH
    sudo git reflog expire --expire=now --expire-unreachable=now --all
    sudo git prune --dry-run --verbose --expire=now
    echo "Git cleanup complete."
    sudo du -hc $GIT_PATH
}

# NOTE: To be removed after one run
cleanup_loki() {
    echo "Initiating loki logs cleanup..."
    sudo du -hc $LOKIDATA_PATH
    sudo rm -rf $LOKIDATA_PATH/*
    echo "Loki logs cleanup complete."
    sudo du -hc $LOKIDATA_PATH
}

# NOTE: To be removed after one run
cleanup_docker() {
    echo "Initiating docker cleanup..."
    cd "$PROJECT_PATH" || exit
    sudo docker system df
    sudo docker compose down
    sudo docker system prune -af --volumes
    echo "Docker cleanup complete."
    sudo docker system df
}

# NOTE: TO be removed after one run
cleanup_log() {
    echo "Initiating log cleanup..."
    cd "$PROJECT_PATH" || exit
    sudo du -hc $SCRIPT_LOG_PATH
    sudo rm -rf $SCRIPT_LOG_PATH
    sudo du -hc $VIMACOG_GEN_LOG_PATH
    sudo rm -rf $VIMACOG_GEN_LOG_PATH
    sudo du -hc $VIMACOG_HARD_GEN_LOG_PATH
    sudo rm -rf $VIMACOG_HARD_GEN_LOG_PATH
    sudo du -hc $ONEAPI_LOG_PATH
    sudo rm -rf $ONEAPI_LOG_PATH
    sudo du -hc $ACCUMULATOR_LOG_PATH
    sudo rm -rf $ACCUMULATOR_LOG_PATH
    sudo du -hc $VIMACOG_GEN_LOG_DIR
    sudo rm -rf $VIMACOG_GEN_LOG_DIR/*
    sudo du -hc $VIMACOG_GEN_BACKUP_DIR
    sudo rm -rf $VIMACOG_GEN_BACKUP_DIR/*
    sudo du -hc $VIMACOG_HARD_GEN_LOG_DIR
    sudo rm -rf $VIMACOG_HARD_GEN_LOG_DIR/*
    sudo du -hc $VIMACOG_HARD_GEN_BACKUP_DIR
    sudo rm -rf $VIMACOG_HARD_GEN_BACKUP_DIR/*
}

# NOTE: Run this temporarily
# cleanup_git
# cleanup_loki
# cleanup_docker
# cleanup_log

# Check for Git updates & pull if updates are available
# Returns whether to hard run or soft run
# Note: Runs always
setup_git() {
    echo "Checking for git updates..."
    cd "$PROJECT_PATH" || exit
    # git config pull.rebase true
    sudo git fetch -fpPv origin $GIT_BRANCH
    if [ "$(git rev-parse HEAD)" != "$(git rev-parse FETCH_HEAD)" ]; then
        # Ignore all local changes(comitted & non-comitted)
        sudo git reset --hard origin/$GIT_BRANCH
        # TODO: Change this to $GIT_BRANCH branch
        sudo git pull -fpv origin $GIT_BRANCH
        echo "Repository updated. Applying updates..."
        # Ensure the scripts are executable after a potential update
        sudo chmod +x "$SCRIPT_PATH"
        sudo chmod +x "$VIMACOG_GEN_PATH"
        sudo chmod +x "$VIMACOG_HARD_GEN_PATH"
        sudo chmod 777 "$GRAFANADATA_PATH"
        sudo chmod 777 "$LOKIDATA_PATH"
        # Hard refresh the setup
        echo "Force updating script due to changes..."
        exec "$SCRIPT_PATH" true
    else
        echo "Repository is up to date."
    fi
}

# Setup environment variables
# `setup_env` updates environment variables in `.env.shepherd` as per updates
# Note: Runs always
setup_env() {

    local force_update="$1"
    echo "Checking for env files..."
    if [ ! -f "$ENV_SHEPHERD_PATH" ]; then
        echo "Env file $ENV_SHEPHERD_PATH not found. Contact support@shepherd.watch in case of queries."
        exit 1
    fi
    if [ ! -f "$ENV_CLIENT_PATH" ]; then
        echo "Env file $ENV_CLIENT_PATH not found. Contact support@shepherd.watch in case of queries."
        exit 1
    fi
    echo "Checking for vimacog env files..."
    if [ ! -f "$ENV_VIMACOG_GEN" ] || [ "$force_update" = true ]; then
        echo "Vimacog env file not found(or force updating), generating $ENV_VIMACOG_GEN..."
        echo "export LOG_FILE_DIR=${VIMACOG_GEN_LOG_DIR}" > "$ENV_VIMACOG_GEN"
        echo "export BACKUP_FILE_DIR=${VIMACOG_GEN_BACKUP_DIR}" >> "$ENV_VIMACOG_GEN"
        echo "export SENTRY_LINK=" >> "$ENV_VIMACOG_GEN"

        JWT_SIGNING_KEY=$(extract_env_value_or_exit "JWT_SIGNING_KEY" "$ENV_SHEPHERD_PATH")
        PUBLIC_TEAMSPACE_ID=$(extract_env_value_or_exit "PUBLIC_TEAMSPACE_ID" "$ENV_SHEPHERD_PATH")
        SHEPHERD_API_JRPC_URL=$(extract_env_value_or_exit "SHEPHERD_API_JRPC_URL" "$ENV_SHEPHERD_PATH")

        echo "export JWT_SIGNING_KEY=$JWT_SIGNING_KEY" >> "$ENV_VIMACOG_GEN"
        echo "export PUBLIC_TEAMSPACE_ID=$PUBLIC_TEAMSPACE_ID" >> "$ENV_VIMACOG_GEN"
        echo "export SHEPHERD_API_JRPC_URL=$SHEPHERD_API_JRPC_URL" >> "$ENV_VIMACOG_GEN"
    fi
    if [ ! -f "$ENV_VIMACOG_HARD_GEN" ] || [ "$force_update" = true ]; then
        echo "Vimacog env file not found(or force updating), generating $ENV_VIMACOG_HARD_GEN..."
        echo "export LOG_FILE_DIR=${VIMACOG_HARD_GEN_LOG_DIR}" > "$ENV_VIMACOG_HARD_GEN"
        echo "export BACKUP_FILE_DIR=${VIMACOG_HARD_GEN_BACKUP_DIR}" >> "$ENV_VIMACOG_HARD_GEN"
        echo "export SENTRY_LINK=" >> "$ENV_VIMACOG_HARD_GEN"

        JWT_SIGNING_KEY=$(extract_env_value_or_exit "JWT_SIGNING_KEY" "$ENV_SHEPHERD_PATH")
        PUBLIC_TEAMSPACE_ID=$(extract_env_value_or_exit "PUBLIC_TEAMSPACE_ID" "$ENV_SHEPHERD_PATH")
        SHEPHERD_API_JRPC_URL=$(extract_env_value_or_exit "SHEPHERD_API_JRPC_URL" "$ENV_SHEPHERD_PATH")

        echo "export JWT_SIGNING_KEY=$JWT_SIGNING_KEY" >> "$ENV_VIMACOG_HARD_GEN"
        echo "export PUBLIC_TEAMSPACE_ID=$PUBLIC_TEAMSPACE_ID" >> "$ENV_VIMACOG_HARD_GEN"
        echo "export SHEPHERD_API_JRPC_URL=$SHEPHERD_API_JRPC_URL" >> "$ENV_VIMACOG_HARD_GEN"
    fi

    # Variables are updated here
    ## Use this function only to update `alertmanager` env, not suitable for other extractions
    ONEAPI_ALERTS_PUSH=$(extract_env_value_or_exit "ONEAPI_ALERTS_PUSH" "$ENV_CLIENT_PATH")
    BASIC_BEARER_TOKEN=$(extract_env_value_or_exit "BASIC_BEARER_TOKEN" "$ENV_CLIENT_PATH")
    echo "$ONEAPI_ALERTS_PUSH" > "$ENV_ALERTMANAGER_URL_PATH"
    echo "$BASIC_BEARER_TOKEN" > "$ENV_ALERTMANAGER_SECRET_PATH"

    # Update this to rollout new update
    update_env_variable "WEBZERO_VERSION" "0.32.1" "$ENV_SHEPHERD_PATH"
    update_env_variable "REPORTS_ACCUMULATOR_VERSION" "0.5.2" "$ENV_SHEPHERD_PATH"
    update_env_variable "REPORTS_CRUNCHER_VERSION" "0.6.2" "$ENV_SHEPHERD_PATH"
    update_env_variable "REPORTS_ENGINE_VERSION" "0.7.0" "$ENV_SHEPHERD_PATH"
    update_env_variable "TAG" "0.8.0" "$ENV_SHEPHERD_PATH"
    update_env_variable "ONEAPI_VERSION" "0.32.1" "$ENV_SHEPHERD_PATH"

    update_env_variable "NANOMDM_SCEP_URL" "http://0.0.0.0:8080" "$ENV_SHEPHERD_PATH"
    update_env_variable "NANOMDM_MDM_URL" "http://0.0.0.0:9001" "$ENV_SHEPHERD_PATH"

    # accessed by reportsengine
    update_env_variable "REPORTS_CRUNCHER_API" "http://reports_cruncher:3000" "$ENV_SHEPHERD_PATH"
    # accessed by dashboard
    update_env_variable "REPORTS_URL" "http://0.0.0.0:3030" "$ENV_SHEPHERD_PATH"
    update_env_variable "APP_DEBUG" "false" "$ENV_SHEPHERD_PATH"

    update_env_variable "NTFY_WS_URL" "ws://0.0.0.0:8003" "$ENV_SHEPHERD_PATH"
    update_env_variable "NTFY_AUTH_TOKEN" "" "$ENV_SHEPHERD_PATH"
    update_env_variable "NTFY_API_URL" "http://0.0.0.0:8003" "$ENV_SHEPHERD_PATH"

    ONEAPI_DOMAIN_NAME=$(extract_env_value_or_exit "ONEAPI_DOMAIN_NAME" "$ENV_CLIENT_PATH")
    update_env_variable "ONEAPI_SERVER_URL" "https://$ONEAPI_DOMAIN_NAME" "$ENV_SHEPHERD_PATH"

    # NOTE: Only need to be updated once
    # ADM required variables
    # echo "NANODEP_URL=" >> "$ENV_CLIENT_PATH"
    # echo "NANODEP_BASIC_AUTH=" >> "$ENV_CLIENT_PATH"

    # Reports required variables
    # echo "SENTRY_URL=" >> "$ENV_SHEPHERD_PATH"
    # echo "SENTRY_ENV=" >> "$ENV_SHEPHERD_PATH"

    # OneAPI required variables
    # echo "NTFY_WS_URL=wss://ntfy.sh" >> "$ENV_SHEPHERD_PATH"
    # echo "NTFY_AUTH_TOKEN=" >> "$ENV_SHEPHERD_PATH"
    # echo "NTFY_API_URL=https://ntfy.sh" >> "$ENV_SHEPHERD_PATH"

    # Dashboard required variables
    # ONE_TOKEN=$(extract_env_value_or_exit "BASIC_BEARER_TOKEN" "$ENV_CLIENT_PATH")
    # echo "ONE_TOKEN=$ONE_TOKEN" >> "$ENV_CLIENT_PATH"

    # Dashboard ADM v2 release required variables
#     echo "ANALYTICS_API_URL=http://reportsanalyzer:3000" >> "$ENV_SHEPHERD_PATH"
#     echo "ONEAPI_VAULT_TOKEN=" >> "$ENV_SHEPHERD_PATH"
#     echo "ANALYTICS_API=http://reportsanalyzer:3000" >> "$ENV_SHEPHERD_PATH"

#     echo "DEPLOYMENT_ENV=production" >> "$ENV_SHEPHERD_PATH"
#     echo "SENTRY_URL=" >> "$ENV_SHEPHERD_PATH"
#     echo "SENTRY_ENV=" >> "$ENV_SHEPHERD_PATH"
#     echo "ENV=production" >> "$ENV_SHEPHERD_PATH"

#     MDM_HOST=$(extract_env_value_or_exit "ADM_DOMAIN_NAME" "$ENV_CLIENT_PATH")
#     echo "MDM_HOST=$MDM_HOST" >> "$ENV_CLIENT_PATH"

#     echo "OIDC_KEYCLOAK_CLIENT_ID=abm-device-login" >> "$ENV_SHEPHERD_PATH"
#     echo "OIDC_KEYCLOAK_CLIENT_SECRET=" >> "$ENV_SHEPHERD_PATH"
#     echo "KEYCLOAK_HOST=login.subcom.link" >> "$ENV_SHEPHERD_PATH"
#     echo "KEYCLOAK_REALM=thriwe-private" >> "$ENV_SHEPHERD_PATH"

#     echo "NANODEP_URL=http://0.0.0.0:9001" >> "$ENV_SHEPHERD_PATH"
#     echo "NANODEP_BASIC_AUTH=" >> "$ENV_SHEPHERD_PATH"

#     echo "MDM_DEVICE_CA=""
# " >> "$ENV_SHEPHERD_PATH"
# echo "MDM_DEVICE_PKEY=""
# " >> "$ENV_SHEPHERD_PATH"

#     update_env_variable "NANOMDM_MDM_URL" "https://0.0.0.0:9002" "$ENV_SHEPHERD_PATH"

#     POSTGRES_DB=$(extract_env_value_or_exit "DB_DATABASE" "$ENV_CLIENT_PATH")
#     echo "POSTGRES_DB=$POSTGRES_DB" >> "$ENV_CLIENT_PATH"
#     POSTGRES_USER=$(extract_env_value_or_exit "DB_USERNAME" "$ENV_CLIENT_PATH")
#     echo "POSTGRES_USER=$POSTGRES_USER" >> "$ENV_CLIENT_PATH"
#     POSTGRES_PASSWORD=$(extract_env_value_or_exit "DB_PASSWORD" "$ENV_CLIENT_PATH")
#     echo "POSTGRES_PASSWORD=$POSTGRES_PASSWORD" >> "$ENV_CLIENT_PATH"

    # Dashboard ADM v2 release required variables for running oneapi
    # echo "SUBCOM_AUTH_PSQL_ENCRYPTION_KEY1=" >> "$ENV_SHEPHERD_PATH"
    # echo "SUBCOM_AUTH_PSQL_ENCRYPTION_KEY2=" >> "$ENV_SHEPHERD_PATH"

    # echo "ENV=production" >> "$ENV_SHEPHERD_PATH"
    # echo "SENTRY_LINK=https://d53976609fd1a93925a0ac256709366f@traces.subcom.link/5" >> "$ENV_SHEPHERD_PATH"
    # echo "SENTRY_DSN=https://d53976609fd1a93925a0ac256709366f@traces.subcom.link/5" >> "$ENV_SHEPHERD_PATH"

    # Caddy file replace
    echo "Generating caddyfile..."
    PORTAL_DOMAIN_NAME=$(extract_env_value_or_exit "PORTAL_DOMAIN_NAME" "$ENV_CLIENT_PATH")
    ONEAPI_DOMAIN_NAME=$(extract_env_value_or_exit "ONEAPI_DOMAIN_NAME" "$ENV_CLIENT_PATH")
    METRICS_DOMAIN_NAME=$(extract_env_value_or_exit "METRICS_DOMAIN_NAME" "$ENV_CLIENT_PATH")
    ADM_DOMAIN_NAME=$(extract_env_value_or_exit "ADM_DOMAIN_NAME" "$ENV_CLIENT_PATH")
    echo "Copying $PORTAL_DOMAIN_NAME, $ONEAPI_DOMAIN_NAME, $METRICS_DOMAIN_NAME, $ADM_DOMAIN_NAME to $CADDYFILE_PATH..."
    replace_variable "PORTAL_DOMAIN_NAME" "$PORTAL_DOMAIN_NAME" "$CADDYFILE_PATH"
    replace_variable "ONEAPI_DOMAIN_NAME" "$ONEAPI_DOMAIN_NAME" "$CADDYFILE_PATH"
    replace_variable "METRICS_DOMAIN_NAME" "$METRICS_DOMAIN_NAME" "$CADDYFILE_PATH"
    replace_variable "ADM_DOMAIN_NAME" "$ADM_DOMAIN_NAME" "$CADDYFILE_PATH"
    echo "Caddyfile generated."

    if [ ! -f "$ENV_PATH" ] || [ "$force_update" = true ]; then
        echo "Env file not found(or force updating), generating..."
        echo "$ENV_PATH not found. Generating..."
        cat ./.env.shepherd ./.env.client > .env
    fi
}

# Setup or update the cron job
# Note: Runs on requirement
setup_cron_job() {
    is_active "cron"

    echo "Checking for cron..."
    if [ "$CRON_EXISTS" -eq "0" ]; then
        # Cron job doesn't exist, do add it
        (crontab -l; echo "$CRON_JOB") | crontab -
        echo "Cron job added: $CRON_JOB"
    else
        echo "Cron job already exists."
    fi

}

# Setup caddy
# Note: Has parts that run always & on requirement
setup_caddy() {
    is_active "caddy"

    local force_update="$1"
    if [ "$force_update" = true ]; then
        echo "Setting up caddy..."
        sudo cp "$CADDYFILE_PATH" /etc/caddy/Caddyfile
        caddy reload --config=/etc/caddy/Caddyfile
    else
        echo "Skipping caddy setup."
    fi
}

# Setup docker containers
# Note: Has parts that run always & on requirement
setup_docker() {
    is_active "docker"

    local force_update="$1"
    cd "$PROJECT_PATH" || exit
    if [ "$force_update" = true ]; then
        echo "Force updating images..."
        sudo docker compose down
        sudo docker volume rm shepherd-enterprise_app
        sudo docker system prune -af --volumes
        sudo docker compose pull >/dev/null
        sudo docker compose --env-file ./.env up -d
    else
        echo "Already running latest version containers."
    fi

    echo "Checking for running docker containers..."
    if sudo docker compose ps | grep -q "Up"; then
        echo "Docker containers are already running."
    else
        echo "Docker containers are not running, trying to restart them..."
        sudo docker compose pull >/dev/null
        sudo docker compose --env-file ./.env up -d
    fi
}

# Setup or update the vimacog
# Note: Runs on requirement
setup_vimacog() {
    is_active "cron"

    echo "Checking for vimacog..."
    if [ "$VIMACOG_GEN_EXISTS" -eq "0" ]; then
        # Cron job doesn't exist, do add it
        (crontab -l; echo "$VIMACOG_GEN_JOB") | crontab -
        echo "Vimacog generation job added: $VIMACOG_GEN_JOB"
    else
        echo "Vimacog generation job already exists."
    fi
    if [ "$VIMACOG_HARD_GEN_EXISTS" -eq "0" ]; then
        # Cron job doesn't exist, do add it
        (crontab -l; echo "$VIMACOG_HARD_GEN_JOB") | crontab -
        echo "Vimacog hard generation job added: $VIMACOG_HARD_GEN_JOB"
    else
        echo "Vimacog hard generation job already exists."
    fi
}

setup_git
setup_env $FORCE
setup_cron_job
setup_caddy $FORCE
setup_docker $FORCE
setup_vimacog

echo -e "\nStorage profile:\n$(df -h)"
echo -e "\nDocker profile:\n$(sudo docker system df)"
echo -e "\nDocker containers profile:\n$(sudo docker ps)"
echo -e "\nDocker ntfy profile:\n$(sudo docker logs -n 100 ntfy)"
echo -e "\nDocker reports accumulator profile:\n$(sudo docker logs -n 100 reportsaccumulator)"
echo -e "\nDocker reports cruncher profile:\n$(sudo docker logs -n 100 reportscruncher)"
echo -e "\nDocker reports engine profile:\n$(sudo docker logs -n 100 reportsengine)"
echo -e "\nDocker portal profile:\n$(sudo docker logs -n 100 portal)"
echo "Setup complete."
exit 0
